//
// See license.txt
//
package lv.tsi.ai;

import java.text.DecimalFormat;
import java.util.Random;

import lv.tsi.util.Log;

/**
 *  ,    
 *        
 *
 * struct PopStr {
 *     char Genome [L + 1];
 *     double Fitness;
 * } Pop[PopSize], NewPop[PopSize];
 *
 * // 
 *
 * GenerateRandomPopulation (Pop);
 * EvaluatePopulation (Pop);
 * while (Not_Done) (
 *     DoTournamentSelection (Pop, NewPop);
 *     ModifyPopulation (NewPop);
 *     Switch (Pop, NewPop);
 *     EvaluatePopulation (Pop);
 * }
 *
 * @version 0.1.0 6 March 2001
 * @author Alexey Vasilyev
 */
public class GeneticAlgorithm {

    /** Mutation probability per bit **/
    protected double mutation_prob_bit = 0.005;

    /** Mutation probability per population **/
    protected double mutation_prob_pop = 0.75;

    /** Number of points to crossover **/
    protected int crossover_points = 1;

    /** Randomizer **/
    protected Random rand = new Random();

    /** Formating float numbers for presentation **/
    static DecimalFormat form = new DecimalFormat();

    /**
     * Generating random population.
     */
    public void GenerateRandomPopulation(PopStrA[] pop) {
        for (int i=0; i<pop.length; i++) {
             for (int j=0; j<pop[i].genome.length; j++)
                 pop[i].genome[j] = rand.nextDouble() > 0.5 ? '0' : '1';
        }
    }

    /**
     *   .
     * EvaluatePopulation       
     * ,      f(x)   
     * ,  . 
     */
    public void EvaluatePopulation(PopStrA[] pop) {
        //System.out.println("Size: " + pop.length + " Pop size:" + pop_size);
        int i;
        double total  = 0;
        double intrvl = 0;
        for (i=0; i<pop.length; i++) {
             pop[i].recalculateFitness();
             total += pop[i].fitness;
        }
        // Setting probabilities
        //for (i=0; i<pop_size; i++) {
        for (i=0; i<pop.length; i++) {
             pop[i].norm = pop[i].fitness / total;
             intrvl += pop[i].norm;
             pop[i].interval = intrvl;
        }
    }

    /**
     *  - ,      
     * .
     *  ModifyPopulation     
     *      NewPop;   
     *  75 (. .  75% ).     
     *  "  ",       
     * .        , 
     *  ,      ,
     *  0  1,  .    
     * 0,5%   -  ,    
     *  ,  0,005.
     */
    public void ModifyPopulation(PopStrA[] newPop) {
        // resorting
        /*for (int k=0; k<pop_size; k++) {
            i = (int)(rand.nextDouble() * pop_size);
            PopStrA p = newPop[k];
            newPop[k] = newPop[i];
            newPop[i] = p;
        }*/
        //System.out.println("Size: " + newPop.length + " pop_size: " + pop_size + " genome: " + genome_size);
        multipointCrossover(newPop);
        mutation (newPop);
    }

    /**
     * Multipoint crossover
     */
    protected void multipointCrossover(PopStrA[] newPop) {
        for (int i=0; i<crossover_points; i++)
            crossover(newPop);
    }

    /**
     * Single point crossover.
     */
    protected void crossover(PopStrA[] newPop) {
        int i, j, k;
        char c;
        for (k=0; k<newPop.length; k=k+2) {
            i = (int)(rand.nextDouble() * newPop[k].genome.length); // tochka razriva
            for (j=0; j<newPop[k].genome.length; j++) {
                if (j>=i) {
                    c = newPop[k].genome[j];
                    newPop[k]  .genome[j] = newPop[k+1].genome[j];
                    newPop[k+1].genome[j] = c;
                }
            }
        }
    }

    /**
     * Mutation.
     */
    protected void mutation(PopStrA[] newPop) {
        int i, j, k;
        // mutation
        // only 75% of the population will be mutated
        for (k=0; k<newPop.length*mutation_prob_pop; k++) {
            /*     Pop */
            i = (int)(rand.nextDouble() * newPop.length);
            //
            for (j=0; j<newPop[i].genome.length; j++) {
                if (rand.nextDouble() <= mutation_prob_bit) {
                //if (rand.nextGaussian() <= 0.005)
                    // inversion
                    newPop[i].genome[j] = (newPop[i].genome[j] == '0' ? '1' : '0');
                    System.out.println("[" +(i+1)+ "] Gen mutated: " + (j+1));
                }
            }
        } 
    }

    /**
     * Reproduction.
     */
    public void DoReproduction(PopStrA[] pop, PopStrA[] newPop) {
        int i, j, k;
        double d;
        for (k=0; k<pop.length; k++) {
            d = rand.nextDouble();
            for (i=0; i<pop.length; i++)
                if (pop[i].interval > d) {
                    newPop[k] = pop[i].createCopy();
                    break;
                }
        }
    }

    /**
     *  .
     *  DoTournamentSelection    
     *      (  )  
     *   (i, j)  NewPop. 
     *
     *  nextGaussian()  ( ) 
     *  0  1.       
     *         
     *   .       
     *       .
     * ,       nextGaussian(),  0.75, 
     *      (  
     *     NewPop),      
     *      .   
     * 0.5,   ,  ,  .
     */
    public void DoTournamentSelection(PopStrA[] pop, PopStrA[] newPop) {
        int  i,  j;
        for (int k=0; k<pop.length; k++) {
        //for (int k=0; k<pop_size; k++) {
            /*      Pop */ 
            i = (int)(rand.nextDouble() * pop.length);
            j = (int)(rand.nextDouble() * pop.length);
            /*     0  1 */ 
            if (rand.nextGaussian()<0.75)
                //    Pop[i], Pop[j]  NewPop
                // in our case vice versa
                newPop[k] = ( pop[i].fitness > pop[j].fitness ? pop[i].createCopy() : pop[j].createCopy() );
            else
                //    Pop[i], Pop[J]  NewPop
                newPop[k] = ( pop[i].fitness < pop[j].fitness ? pop[i].createCopy() : pop[j].createCopy() );
        }
    }

    public void Switch (PopStrA[] pop, PopStrA[] newPop) {
        for (int i=0; i<pop.length; i++)
            pop[i] = newPop[i];
    }

    /**
     * Dump genomes state to stdout.
     */
    public void Dump(PopStrA[] pop) {
        double total = 0;
        double max   = 0;
        for (int i=0; i<pop.length; i++) {
            Log.debug( "[" + (i+1) + "] " + pop[i] );
            total += pop[i].fitness;
            if (pop[i].fitness > max)
                max = pop[i].fitness;
        }
        Log.debug("Max: " + max + "  Avg: " + total/pop.length+ "  Total: " + total);
    }

    public void setMutationProbabilityPerBit(double mut) {
        mutation_prob_bit = mut;
    }
    public double getMutationProbabilityPerBit() { return mutation_prob_bit; }

    public void setMutationProbabilityPerPop(double mut) {
        mutation_prob_pop = mut;
    }
    public double getMutationProbabilityPerPop() { return mutation_prob_pop; }
    public int    getNumberOfCrossoverPoints()   { return crossover_points;  }
    public void setNumberOfCrossoverPoints(int num) { crossover_points = num; }

}
