//
// See license.txt
//
package lv.tsi.ai;

/**
 * Instance of the population.
 * All descendants should implements following abstract functions:
 *   protected abstract PopStrA createInstance();
 *   public abstract void recalculateFitness();
 *
 * @version 0.1.0 6 March 2001
 * @author Alexey Vasilyev
 */
public abstract class PopStrA implements Cloneable, java.io.Serializable {

    public char[] genome;       // x
    public double fitness = 0;  // function f(x) value
    double norm = 0;            // Normalized fitness
    double interval = 0;        // upper interval boundary

    public PopStrA(int genome_size) {
        genome = new char[genome_size];
    }

    protected abstract PopStrA createInstance();

    public PopStrA createCopy() { 
        try { 
            PopStrA p = createInstance();
            p.genome = (char[])genome.clone();
            p.norm = norm; p.interval = interval; p.fitness = fitness;
            return p;
        } catch (Exception e) { return null;}
    }

    public abstract void recalculateFitness();

    public String toString() {
        return 
          "Fitness: "  + GeneticAlgorithm.form.format( fitness )
        + "   Normalized: " + GeneticAlgorithm.form.format( norm )
        + "   Interval: " + GeneticAlgorithm.form.format( interval )
        + "   Gen: " + new String(genome);
    }

}
