//
// See license.txt
//
package lv.tsi.ai;

/**
 * Music specific.
 *
 * @version 0.1.0 6 March 2001
 * @author Alexey Vasilyev
 */
public class PopStrMusic extends PopStrA implements Cloneable {

    public PopStrMusic(int genome_size) {
        super(genome_size);
    }

    protected PopStrA createInstance() {
        return new PopStrMusic(genome.length);
    }

    public void recalculateFitness() {
        // Do nothing
    }

    public int[] getNotes() {
        return mapStringToNotes(genome);
    }

    /**
     * @param S char array of 1 and 0. For example, '1','0','0','1','1','1'
     *
     * @return array of notes
     */
    public static int[] mapStringToNotes (char[] S) {
        //    S    r    
        int r, i, j = 0;

        int[] ret = new int[ S.length / bites_per_note ];
        for (r=i=j=0; i<S.length; ) {
             r<<=1;           /*   ,  0    */ 
             if (S[i++]=='1') /*   S 1,  ... */ 
                 ++r;         /*      1 */
             // Start new note 
             if (i % bites_per_note == 0) {
                 if (r<0) System.out.println("Error:" + r);
                 ret[j++] = r;
                 r = 0;
             }
        }
        return ret;
    }


    /**
     * @param S   char array of notes. For example, 12,3,5,7,9,1
     *
     * @return array of chars represented by genome, i.e. '1','0','1','1','1'
     */
    public static char[] mapNotesToString (int[] S) {
        int i, j, k;

        char[] ret = new char[ S.length * bites_per_note ];
        for (i=0; i<S.length; i++) {
            for (j=0; j<bites_per_note; j++ ) {
                k = i*bites_per_note + bites_per_note-j-1;
                ret[k] = ((S[i] & 0x1) == 1) ? '1' : '0';
                S[i]>>=1;         /*    */ 
            }
        }
        return ret;
    }

    /**
     * @return string with notes. For example, "F# GDC C#"
     */
    public static String mapIntToNoteNames (int[] in) {
        StringBuffer buf = new StringBuffer(in.length);
        for (int i=0; i<in.length; i++) {
            buf.append( NOTES[ in[i] ] );
        }
        return buf.toString();
    }

    public String toString() {
        return 
          "Fitness: "  + GeneticAlgorithm.form.format( fitness )
        + "   Notes: " + mapIntToNoteNames(mapStringToNotes(genome))
        + "\t Normalized: " + GeneticAlgorithm.form.format( norm )
        + "   Interval: " + GeneticAlgorithm.form.format( interval )
        + "   Gen: " + new String(genome);
    }

    /** Number of bites symbolize 1 note **/
    static int bites_per_note = 4;

    public final static String[] NOTES = { "C", "C#", "D", "D#", "E", "F", "F#",
     "G", "G#", "A", "A#", "B", " ", " ", " ", " " };

}
