//
// See license.txt
//
package lv.tsi.graphics;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.sound.midi.MidiSystem;
import java.io.*;

import lv.tsi.util.*;

/**
 * Main class for starting GUI framework.
 *
 * @version 0.3.2 12 March 2001
 * @author Alexey Vasilyev
 */
public class ComposerUI extends JPanel {

    /** MIDI processor **/
    static SynthNote synth;
    /** GA processor **/
    public static MusicProducer prod;
    /** Where table is stored **/
    static IGAPanel pnlIGA;
    /** Options panel **/
    static OptionsPanel pnlOptions;

    static Icon imgInfo, imgMenuNew, imgMenuOpen, imgMenuSave, imgMenuExit, 
                imgMenuPlay, imgMenuStartR, imgMenuStopR, imgMenuAbout ;

    JTabbedPane tabPane = new JTabbedPane();
    int width = 760, height = 500;
    int index;

    public ComposerUI() {
        super(new BorderLayout());
        try {
            imgInfo = new ImageIcon( this.getClass().getResource("/lv/tsi/images/info.gif") );
        } catch (Exception e) {}
        try {
            imgMenuNew = new ImageIcon( this.getClass().getResource("/lv/tsi/images/menu_new.gif") );
        } catch (Exception e) {}
        try {
            imgMenuOpen = new ImageIcon( this.getClass().getResource("/lv/tsi/images/menu_open.gif") );
        } catch (Exception e) {}
        try {
            imgMenuSave = new ImageIcon( this.getClass().getResource("/lv/tsi/images/menu_save.gif") );
        } catch (Exception e) {}
        try {
            imgMenuExit = new ImageIcon( this.getClass().getResource("/lv/tsi/images/menu_exit.gif") );
        } catch (Exception e) {}
        try {
            imgMenuPlay = new ImageIcon( this.getClass().getResource("/lv/tsi/images/menu_play.gif") );
        } catch (Exception e) {}
        try {
            imgMenuStartR = new ImageIcon( this.getClass().getResource("/lv/tsi/images/menu_startrecord.gif") );
        } catch (Exception e) {}
        try {
            imgMenuStopR = new ImageIcon( this.getClass().getResource("/lv/tsi/images/menu_stoprecord.gif") );
        } catch (Exception e) {}
        try {
            imgMenuAbout = new ImageIcon( this.getClass().getResource("/lv/tsi/images/menu_about.gif") );
        } catch (Exception e) {}

        JMenuBar menuBar = new JMenuBar();
       
        JMenu fileMenu = (JMenu) menuBar.add(new JMenu("File"));
        fileMenu.setMnemonic('F');
        JMenuItem item = (JMenuItem) fileMenu.add(new JMenuItem("New random generation", imgMenuNew));
        item.setMnemonic('e');
        item.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                ComposerUI.prod  = new MusicProducer();
                ComposerUI.pnlIGA.refreshGeneration();
            }
        });
 	fileMenu.add( new JSeparator() );
        item = (JMenuItem) fileMenu.add(new JMenuItem("Load generation ...", imgMenuOpen) );
        item.setMnemonic('L');
        //  ------------------------------------------
        // | Loading as serialized form generation
        //  ------------------------------------------
        item.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                try {
                    JFileChooser fc = new JFileChooser(new File(System.getProperty("user.dir")));
                    fc.setFileFilter(new javax.swing.filechooser.FileFilter() {
                        public boolean accept(File f) {
                            if (f.isDirectory() || f.getName().endsWith(".gen")) {
                                return true;
                            }
                            return false;
                        }
                        public String getDescription() {
                            return "Load as .gen file.";
                        }
                    });
                    if (fc.showOpenDialog(null) == JFileChooser.APPROVE_OPTION) {
                        ObjectSerializer.load( fc.getSelectedFile() );
                        System.out.println("Generation loaded. Pop size: " + ComposerUI.prod.pop_size + " Genome size: " + ComposerUI.prod.genome_size);
                        ComposerUI.pnlIGA.refreshGeneration();
                        ComposerUI.pnlOptions.init();
                    }
                } catch (Exception ex) {
                    System.out.println("Can't load generation due to: " + ex.getMessage() );
                }
            }
        });
        item = (JMenuItem) fileMenu.add(new JMenuItem("Load melody to process ...", imgMenuOpen));
        item.setMnemonic('m');
        //  ------------------------------------------
        // | Loading as plain text generation
        //  ------------------------------------------
        item.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                try {
                    JFileChooser fc = new JFileChooser(new File(System.getProperty("user.dir")));
                    fc.setFileFilter(new javax.swing.filechooser.FileFilter() {
                        public boolean accept(File f) {
                            if (f.isDirectory() || f.getName().endsWith(".tab")) {
                                return true;
                            }
                            return false;
                        }
                        public String getDescription() {
                            return "Load as .tab file.";
                        }
                    });
                    if (fc.showOpenDialog(null) == JFileChooser.APPROVE_OPTION) {
                        FileParser.load( fc.getSelectedFile() );
                        System.out.println("Melody loaded. Pop size: " + ComposerUI.prod.pop_size + " Genome size: " + ComposerUI.prod.genome_size);
                        ComposerUI.pnlIGA.refreshGeneration();
                        ComposerUI.pnlOptions.init();
                    }
                } catch (Exception ex) {
                    System.out.println("Can't load melody due to: " + ex.getMessage() );
                }
            }
        });
 	fileMenu.add( new JSeparator() );
        item = (JMenuItem) fileMenu.add(new JMenuItem("Save current generation ...", imgMenuSave));
        item.setMnemonic('S');
        //  ------------------------------------------
        // | Saving as serialized form
        //  ------------------------------------------
        item.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                try {
                    JFileChooser fc = new JFileChooser(new File(System.getProperty("user.dir")));
                    fc.setFileFilter(new javax.swing.filechooser.FileFilter() {
                        public boolean accept(File f) {
                            if (f.isDirectory() || f.getName().endsWith(".gen")) {
                                return true;
                            }
                            return false;
                        }
                        public String getDescription() {
                            return "Save as .gen file.";
                        }
                    });
                    if (fc.showSaveDialog(null) == JFileChooser.APPROVE_OPTION) {
                        ObjectSerializer.save( fc.getSelectedFile() );
                        System.out.println("Generation saved.");
                    }
                } catch (Exception ex) {
                    System.out.println("Can't save generation due to: " + ex.getMessage());
                }
            }
        });
        item = (JMenuItem) fileMenu.add(new JMenuItem("Save last played melody ...", imgMenuSave));
        item.setMnemonic('S');
        //  ------------------------------------------
        // | Saving as .mid file
        //  ------------------------------------------
        item.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                try {
                    JFileChooser fc = new JFileChooser(new File(System.getProperty("user.dir")));
                    fc.setFileFilter(new javax.swing.filechooser.FileFilter() {
                        public boolean accept(File f) {
                            if (f.isDirectory() || f.getName().endsWith(".mid")) {
                                return true;
                            }
                            return false;
                        }
                        public String getDescription() {
                            return "Save as .mid file.";
                        }
                    });
                    if (synth.sequence==null)
                       throw new IOException("The melody hasn't been played before.");
                    if (fc.showSaveDialog(null) == JFileChooser.APPROVE_OPTION) {
                        int[] fileTypes = MidiSystem.getMidiFileTypes(synth.sequence);
                        if (fileTypes.length == 0) {
                            System.out.println("Can't save sequence");
                        } else {
                            if (MidiSystem.write(synth.sequence, fileTypes[0], fc.getSelectedFile()) == -1)
                                throw new IOException("Problems writing to file");
                            else
                                System.out.println("Sequence saved.");
                        }
                    }
                } catch (Exception ex) {
                    System.out.println("Can't save sequence due to: " + ex.getMessage());
                }
            }
        });
 	fileMenu.add( new JSeparator() );
        item = (JMenuItem) fileMenu.add(new JMenuItem("Exit", imgMenuExit));
        item.setMnemonic('x');
        item.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) { ComposerUI.synth.close(); System.exit(0); }
        });
        // Record menu
        JMenu recMenu = (JMenu) menuBar.add(new JMenu("Record"));
        recMenu.setMnemonic('R');
        item = (JMenuItem) recMenu.add(new JMenuItem("Start recording", imgMenuStartR));
        item.setMnemonic('S');
        item.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                JMenuItem i = (JMenuItem)e.getSource();
                boolean recording = !synth.isRecording();
                if (recording) {
                    synth.startRecording();
                    i.setText("Stop recording");
                    i.setIcon( imgMenuStopR );
                } else {
                    synth.stopRecording();
                    i.setText("Start recording");
                    i.setIcon( imgMenuStartR );
                    JOptionPane.showMessageDialog(ComposerUI.this, 
                        "The melody successfully recorded.\n"
                       + "To save melody you have to play it (Play recorded melody) and then\n"
                       + "save (Save last played melody).", "Melody recorded", JOptionPane.INFORMATION_MESSAGE);
                }
            }
        });
        item = (JMenuItem) recMenu.add(new JMenuItem("Play recorded melody", imgMenuPlay));
        item.setMnemonic('P');
        item.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                synth.playRecorded();
            }
        });
        // Help menu
        JMenu helpMenu = (JMenu) menuBar.add(new JMenu("Help"));
        helpMenu.setMnemonic('H');
        item = (JMenuItem) helpMenu.add(new JMenuItem("About ...", imgMenuAbout));
        item.setMnemonic('A');
        item.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                JOptionPane.showMessageDialog(ComposerUI.this, 
                    "GenAMusic v0.3\n"
                  + "Genetic algorithm based music composer.\n\n"
                  + "Written by Alexey Vasilyev (servent@apollo.lv)\n"
                  + "TSI 2001 (www.tsi.lv), Riga, Latvia", "Composer about", JOptionPane.INFORMATION_MESSAGE, ComposerUI.imgInfo);
            }
        });
        add(menuBar, BorderLayout.NORTH);

        EmptyBorder eb = new EmptyBorder(5,5,5,5);
        BevelBorder bb = new BevelBorder(BevelBorder.LOWERED);
        JPanel p = new JPanel(new BorderLayout());

        pnlIGA = new IGAPanel();
        p.add(pnlIGA);
        tabPane.addTab("Interactive GA", p);

        pnlOptions = new OptionsPanel();
        p = new JPanel(new BorderLayout());
        p.add(pnlOptions);
        tabPane.addTab("Options", p);

        add(tabPane, BorderLayout.CENTER);
    }

///////////////////////////////////////
    public static void main(String[] args) {

        // The first parameter can be file name to log debug info
        if (args.length>0)
            Log.init (args[0], Log.DBG);
        else
            Log.init ("composer.log", Log.DBG);

      ////////////
        ComposerUI.synth = new SynthNote();
        ComposerUI.prod  = new MusicProducer();
      ////////////

        final ComposerUI ui = new ComposerUI();
        JFrame f = new JFrame("GenAMusic: Music Composer based on Interactive Genetic Algorithm (IGA)");
        f.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) { ComposerUI.synth.close(); System.exit(0); }
        });
        f.getContentPane().add("Center", ui);
        f.pack();
        Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
        f.setLocation( (int)(d.width/1.5 - ui.width/1.5), (int)(d.height/1.5 - ui.height/1.5) );
        f.setSize(new Dimension(ui.width, ui.height));
        f.setVisible(true);
    }

}
