//
// See license.txt
//
package lv.tsi.graphics;

import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.table.*;
import javax.swing.event.*;
import java.awt.event.*;

import lv.tsi.ai.*;

/**
 * Panel where main table is located.
 *
 * @version 0.3.0 9 March 2001
 * @author Alexey Vasilyev  
 */
public class IGAPanel extends JPanel {

    JTable    tblIGA;
    NextPanel pnlNext;

    static ImageIcon imgPlay;

    public IGAPanel() {
        super(new BorderLayout());
        setBorder(new EmptyBorder(5,5,5,5));
        try {
            imgPlay = new ImageIcon( this.getClass().getResource("/lv/tsi/images/play.gif") );
        } catch (Exception e) {}

        tblIGA = new JTable();

        JScrollPane s = new JScrollPane(tblIGA);
        add(BorderLayout.CENTER, s);
        add(BorderLayout.SOUTH, pnlNext = new NextPanel());

        refreshGeneration();
    }

    /**
     * We need to call each time new TableModel creation since
     * unpredictable behaiviour occures on JButton.
     */
    private void refreshTable() {
        String[] names = { "#", "Genome", "Notes", "Fitness", "Music" };
    
        TableModel dataModel = new DefaultTableModel( names, ComposerUI.prod.pop_size ) {
                public Object getValueAt(int row, int col) { 
                    if (col == 0) {
                        return new Integer(row+1);
                    }
                    return super.getValueAt(row, col);
                }
                public boolean isCellEditable(int row, int col) {
                    if (col==0) return false;
                    return true;
                }
        };

        DefaultTableCellRenderer renderer = new DefaultTableCellRenderer() {
            public Component getTableCellRendererComponent(
                    JTable table, Object val, boolean isSel,
                    boolean hasF, int row, int col) {
                switch (col) {
                    // "Fitness" column
                    case 3: PopStrMusic pop = ((WrapperA)val).pop;
                            JProgressBar p = new JProgressBar(JProgressBar.HORIZONTAL, 0, 10);
                            p.setValue ( (int)pop.fitness );
                            JPanel pnl = new JPanel();
                            pnl.add( p );
                            pnl.setToolTipText("Evaluate melody fitness");
                            return pnl;
                    // "Music" column
                    case 4: JButton btn; 
                            if (imgPlay!=null) btn = new JButton(imgPlay);
                            else btn = new JButton("Play");
                            btn.setToolTipText("Play melody");
                            return btn;
                }
                return super.getTableCellRendererComponent(table, val, isSel, hasF, row, col);
            }
        };

        tblIGA.setModel( dataModel );
        tblIGA.getColumn("#").setMaxWidth(20);
        tblIGA.getColumn("Music").setMaxWidth(100);
        tblIGA.setRowHeight(35);

        IGACellEditor editor = new IGACellEditor();
        editor.setClickCountToStart(1);

        tblIGA.setDefaultRenderer( tblIGA.getColumnClass(0), renderer );
        //tblIGA.setDefaultEditor  ( tblIGA.getColumnClass(0), editor   );
        tblIGA.getColumn("Fitness").setCellEditor(editor);
        tblIGA.getColumn("Music").setCellEditor  (editor);
    }

    void refreshGeneration() {
        refreshTable();
        ((DefaultTableModel)tblIGA.getModel()).setRowCount( ComposerUI.prod.pop_size );
        WrapperMain f;
        for (int i=0; i<ComposerUI.prod.pop_size; i++) {
            tblIGA.getModel().setValueAt(new WrapperGenome (ComposerUI.prod.pop[i]), i, 1);
            tblIGA.getModel().setValueAt(new WrapperNotes  (ComposerUI.prod.pop[i]), i, 2);
            f = new WrapperMain(ComposerUI.prod.pop[i]);
            tblIGA.getModel().setValueAt(f, i, 3);
            f = new WrapperMain(ComposerUI.prod.pop[i]);
            tblIGA.getModel().setValueAt(f, i, 4);
        }
        pnlNext.updateGeneration();
    }

} 

/////////////////////////////////
class IGACellEditor extends DefaultCellEditor implements ChangeListener, ActionListener {

            private JSlider slider = new JSlider( new DefaultBoundedRangeModel(0,1,0,11) );
            private JButton button;
            private JPanel  panel  = new JPanel();
            private PopStrMusic pop;
            private WrapperMain wr;

            public IGACellEditor() {
                super( new JTextField() );
                if (IGAPanel.imgPlay!=null)
                     button = new JButton( IGAPanel.imgPlay );
                else button = new JButton( "Play" );
                slider.addChangeListener( this );
                button.addActionListener( this );
                slider.setMinorTickSpacing(1);
                slider.setMajorTickSpacing(10);
                slider.setPaintTicks( true );
                //slider.setPaintTrack( true );
                panel.add( slider );

                // swing hack
                delegate = new EditorDelegate() {
                    public void setValue(Object val) {
	        	slider.setValue((val != null) ? (int)((WrapperMain)val).pop.fitness : 0);
                    }
        	    public Object getCellEditorValue() {
        		return wr;
        	    }
                };
            }

            // Slider changed
            public void stateChanged(ChangeEvent e) {
                if (e.getSource()==slider && pop!=null) {
                    pop.fitness = slider.getValue();
                    delegate.setValue( wr );
                }
            }

            // Button pressed
            public void actionPerformed(ActionEvent e) {
                //System.out.print("  Pop: " + pop);
                ComposerUI.synth.playMelody( pop.getNotes() );
            }

            public Component getTableCellEditorComponent(
                        JTable table, Object val, boolean isSel,
                        int row, int col) {
                wr = (WrapperMain)val;
                pop = ((WrapperA)val).pop;
                switch (col) {
                    case 3: return panel;  // "Fitness" column
                    case 4: return button; // "Music" column
                }
                return super.getTableCellEditorComponent(table, val, isSel, row, col);
            }

}


/////////////////////////////////
abstract class WrapperA {

    PopStrMusic pop;

    WrapperA(PopStrMusic pop) {
        this.pop = pop;
    }

    public abstract String toString();
}

/////////////////////////////////
class WrapperGenome extends WrapperA {

    WrapperGenome(PopStrMusic pop) {
        super( pop );
    }

    public String toString() {
        return new String(pop.genome);
    }
}

/////////////////////////////////
class WrapperNotes extends WrapperA {

    WrapperNotes(PopStrMusic pop) {
        super( pop );
    }

    public String toString() {
        return PopStrMusic.mapIntToNoteNames(pop.getNotes());
    }
}

/////////////////////////////////
class WrapperMain extends WrapperA {

    WrapperMain(PopStrMusic pop) {
        super( pop );
    }

    public String toString() {
        return Integer.toString( (int)pop.fitness );
    }
}
