//
// See license.txt
//
package lv.tsi.graphics;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

import lv.tsi.ai.*;
import lv.tsi.util.Log;

/**
 * Options tab.
 *
 * @version 0.3.0 9 March 2001
 * @author Alexey Vasilyev  
 */
public class OptionsPanel extends JPanel implements ActionListener {

    JTextField txtGenome          = new JTextField();
    JTextField txtPopulation      = new JTextField();
    JTextField txtMutatProb       = new JTextField();
    JTextField txtMutatProbPerBit = new JTextField();
    JTextField txtMutatProbPerPop = new JTextField();
    JTextField txtCrossoverPoints = new JTextField();

    JTextField txtVelocity   = new JTextField();
    JTextField txtDuration   = new JTextField();
    JTextField txtDelta      = new JTextField();

    JButton btnNew;
    JButton btnApplyGA;
    JButton btnApplyMusic;

    public OptionsPanel() {
        super(new BorderLayout());
        setBorder(new EmptyBorder(5,5,5,5));

        try {
            btnNew = new JButton("New", new ImageIcon( this.getClass().getResource("/lv/tsi/images/new.gif")) ); 
        } catch (Exception e) {
            btnNew = new JButton("New");
        }
        try {
            btnApplyGA = new JButton("Apply", new ImageIcon( this.getClass().getResource("/lv/tsi/images/apply.gif")) ); 
        } catch (Exception e) {
            btnApplyGA = new JButton("Apply");
        }
        try {
            btnApplyMusic = new JButton("Apply", new ImageIcon( this.getClass().getResource("/lv/tsi/images/apply.gif")) ); 
        } catch (Exception e) {
            btnApplyMusic = new JButton("Apply");
        }
        btnNew.setMnemonic('e');
        btnApplyMusic.setMnemonic('A');
        btnApplyGA.setMnemonic('p');

        JPanel p, p2;
        TitledBorder b;
        Dimension d = new Dimension(100,25);

        b = new TitledBorder( new EtchedBorder(), "Music" );
        b.setTitleColor(Color.darkGray);
            txtDuration.setPreferredSize(d);
            txtVelocity.setPreferredSize(d);
            txtDuration.setPreferredSize(d);
            txtDelta.setPreferredSize(d);
        p = new JPanel( new GridLayout(7,2,5,5) );
        p.setBorder( b );
        p.add( new JLabel("  Velocity (0..127)") );
        p.add( txtVelocity );
        p.add( new JLabel("  Note duration") );
        p.add( txtDuration );
        p.add( new JLabel("  MIDI notes delta (0..127) 60-C") );
        p.add( txtDelta );
        p.add( new JLabel() );
        p.add( btnApplyMusic );
        p.add( new JLabel() );
        p.add( new JLabel() );
        p.add( new JLabel() );
        p.add( new JLabel() );
        p.add( new JLabel() );
        p.add( new JLabel() );

        p2 = new JPanel( new GridLayout(1,2,5,5) );
        p2.add(p);

        b = new TitledBorder( new EtchedBorder(), "Genetic Algorithm" );
        b.setTitleColor(Color.darkGray);
            txtGenome.setPreferredSize(d);
            txtPopulation.setPreferredSize(d);
            txtMutatProbPerBit.setPreferredSize(d);
            txtMutatProbPerPop.setPreferredSize(d);
        p = new JPanel( new GridLayout(7,2,5,5) );
        p.setBorder( b );
        p.add( new JLabel("  Notes per genome") );
        p.add( txtGenome );
        p.add( new JLabel("  Population") );
        p.add( txtPopulation );
        p.add( new JLabel() );
        p.add( btnNew );
        p.add( new JLabel("  Mutation prob. per bit") );
        p.add( txtMutatProbPerBit );
        p.add( new JLabel("  Mutation prob. per population") );
        p.add( txtMutatProbPerPop );
        p.add( new JLabel("  Number of crossover points") );
        p.add( txtCrossoverPoints );
        p.add( new JLabel() );
        p.add( btnApplyGA );

        p2.add(p);

        p = new JPanel( new BorderLayout() );
        p.add( BorderLayout.NORTH, p2 );
        add( BorderLayout.WEST, p );

            JLabel tmp = new JLabel("TSI 2001, Alexey Vasilyev");
            tmp.setFont( tmp.getFont().deriveFont( Font.ITALIC+Font.BOLD, 15 ) );
            tmp.setForeground(Color.gray);
        p = new JPanel( new BorderLayout() );
        p.add( BorderLayout.WEST, tmp );
        add( BorderLayout.SOUTH, p );

        btnNew.addActionListener       (this);
        btnApplyGA.addActionListener   (this);
        btnApplyMusic.addActionListener(this);
        init();
    }

    public void init() {
        txtDuration.setText( Integer.toString(ComposerUI.synth.duration) );
        txtDelta.setText( Integer.toString(ComposerUI.synth.notes_delta) );
        txtVelocity.setText( Integer.toString(ComposerUI.synth.velocity) );

        txtGenome.setText( Integer.toString(ComposerUI.prod.genome_size/4) );
        txtPopulation.setText( Integer.toString(ComposerUI.prod.pop_size) );
        txtMutatProbPerBit.setText( Double.toString(ComposerUI.prod.iga.getMutationProbabilityPerBit()) );
        txtMutatProbPerPop.setText( Double.toString(ComposerUI.prod.iga.getMutationProbabilityPerPop()) );
        txtCrossoverPoints.setText( Integer.toString(ComposerUI.prod.iga.getNumberOfCrossoverPoints()) );
    }

    public void actionPerformed(ActionEvent e) {
        try {
            // Button new
            if (e.getSource() == btnNew) {
                int genome_size = Integer.parseInt( txtGenome.getText() ) * 4;
                int pop_size    = Integer.parseInt( txtPopulation.getText() );
                //     
                pop_size    = pop_size / 2 * 2;
                ComposerUI.prod = new MusicProducer(pop_size, genome_size);
                ComposerUI.pnlIGA.refreshGeneration();
            // Button Apply Music
            } else if (e.getSource() == btnApplyMusic) {
                ComposerUI.synth.velocity = Integer.parseInt( txtVelocity.getText() );
                ComposerUI.synth.duration = Integer.parseInt( txtDuration.getText() );
                ComposerUI.synth.notes_delta = Integer.parseInt( txtDelta.getText() );
            // Button Apply GA
            } else if (e.getSource() == btnApplyGA) {
                ComposerUI.prod.iga.setMutationProbabilityPerBit( Double.parseDouble( txtMutatProbPerBit.getText() ) );
                ComposerUI.prod.iga.setMutationProbabilityPerPop( Double.parseDouble( txtMutatProbPerPop.getText() ) );
                ComposerUI.prod.iga.setNumberOfCrossoverPoints( Integer.parseInt( txtCrossoverPoints.getText() ) );
            }
        } catch (Exception ex) {
            Log.error( ex.toString() );
        }
   }

}