//
// See license.txt
//
package lv.tsi.util;

import lv.tsi.graphics.*;
import lv.tsi.ai.*;

import java.io.*;
import java.util.StringTokenizer;

/**
 * Class for parsing incoming text file.
 *
 * @version 0.3.0 9 March 2001
 * @author  Alexey Vasilyev
 */
public class FileParser {

    private static final int HEADER_MELODY_NUMBER = 0;
    private static final int HEADER_MELODY_SIZE   = 1;
    private static final int BODY                 = 2;

    /**
     * @param name       the name of the database (or it's url).
     * @throws           IOException if the database doesn't
     *                   exist.
     */
    public static void load(File fl) throws IOException, NumberFormatException {
        //
        BufferedReader r = new BufferedReader(new FileReader(fl) );

        PopStrMusic[] newPop = new PopStrMusic[1];
        int j = 0;
        int genome_size = 0;
        int pop_size    = 0;
        int parse_type  = HEADER_MELODY_NUMBER;

        ////////////////////////////////////////////////////
        while (r.ready()) {

            String s = r.readLine();

            ////////////////////////////////////////////////////
            if (s!=null) {
                int[] genome = new int[genome_size];
                int i = 0;
                boolean b = false;

                StringTokenizer token = new StringTokenizer(s, ";");


                while (token.hasMoreTokens()) {
                    String s2 = token.nextToken();

                    if (s2.indexOf("//") == -1) {

                        switch (parse_type) {

                            case HEADER_MELODY_NUMBER:
                                 pop_size = Integer.parseInt( s2 );
                                 newPop = new PopStrMusic[ pop_size ];
                                 parse_type = HEADER_MELODY_SIZE;
                                 break;

                            case HEADER_MELODY_SIZE:
                                 genome_size = Integer.parseInt( s2 );
                                 parse_type = BODY;
                                 break;

                            case BODY:
                                 genome[i++] = traverse(s2);
                                 b = true;
                                 break;
                        }
                    } else break;
                }
                if (b) {
                    newPop[j] = new PopStrMusic( pop_size );
                    newPop[j].genome = PopStrMusic.mapNotesToString( genome );
                    ++j;
                }
            }
            ////////////////////////////////////////////////////

        }
        r.close();
        ComposerUI.prod = new MusicProducer(pop_size, 4*genome_size);
        ComposerUI.prod.pop = newPop;
        //for (int i=0; i<6; i++)
        //    System.out.println( newPop[i].genome ) ;
        ////////////////////////////////////////////////////
    }

    private static int traverse(String note)  throws IOException {
        for (int i=0; i<PopStrMusic.NOTES.length; i++) {
            if (note.equals( PopStrMusic.NOTES[i] )) {
                return i;
            }
        }
        throw new IOException("The note '" + note + "' isn't correct.");
    }

}