//
// See license.txt
//
package lv.tsi.util;

import lv.tsi.graphics.*;

import java.io.*;

/**
 * Class for serializing objects states.
 *
 * @version 0.3.1 12 March 2001
 * @author  Alexey Vasilyev
 */
public class ObjectSerializer {

    private static final String FILE_HEADER = "GA";
    private static final String FILE_EXT = ".gen";

    /**
     *
     * @param fl   file where serialized objects will be stored.
     * @throws     IOException if something wrong
     */
    public static void save(File fl) throws IOException {
            // The previous will be overwritten
            ObjectOutputStream oos = new ObjectOutputStream(
                new FileOutputStream( fl )
            );
            // Setting header of the file
            oos.write(FILE_HEADER.getBytes());

            // Saving persistent object
            oos.writeObject(ComposerUI.prod);

            oos.close();
    }

    /**
     * Load all serialized objects.
     *
     * @param fl   file from where load all serialized objects.
     * @throws     IOException if file doesn't exist
     */
    public static void load(File fl) throws IOException {
        try {
            ObjectInputStream ois = new ObjectInputStream(
                new FileInputStream( fl )
            );
            // Checking header of the file
            byte[] header = new byte[FILE_HEADER.length()];
            ois.read(header, 0, header.length);
            if (!new String(header).equals( FILE_HEADER ))
                throw new IOException( "File '" + fl
                    + "' isn't correct. " + new String(header) + FILE_HEADER);

            // Inserting all serialized objects only within opened transaction
            while (true) {
                try {
                    // trying to read objects from stream until EOFException occurs
                    Object obj = ois.readObject();
                    if (obj instanceof MusicProducer) {
                        ComposerUI.prod = (MusicProducer)obj;
              System.out.println("Done");
                    }
                } catch (EOFException e) {
                    break;
                }
            }

        } catch (Exception e) {
            throw new IOException( e.toString() );
        }
    }


}